<?php

namespace App\Http\Controllers\Admin\Structure;

use App\Filter\JobTitle;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CreateJobRequest;
use App\Http\Requests\UpdateJobRequest;
use App\Models\Job;
use App\Models\Skill;
use App\Models\SkillJob;
use App\Repositories\Admin\Structure\FAQRepository;
use App\Repositories\JobRepository;
use App\Repositories\JobTitleRepository;
use App\Traits\ApiResponser;
use App\Traits\pipeline;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

class JobController extends Controller
{

    use ApiResponser,pipeline;

    public $request;
    public $repository;

    public function __construct(Request $request, JobRepository $repository)
    {
        $this->repository = $repository;
        $this->request = $request;
        $this->middleware(['auth:sanctum', 'isAdmin']);

        $this->middleware('create-job')->only(['store']);
        $this->middleware('can:edit-job')->only(['update']);
        $this->middleware('can:read-job')->only(['index']);
        $this->middleware('can:delete-job')->only(['delete']);
    }

    public function store(CreateJobRequest $jobRequest)
    {

        if (array_key_exists('job', $jobRequest->validated())) {
            $jobTitleRepository = new JobTitleRepository();
            $job = $jobTitleRepository->findJob($jobRequest->validated()['job']);
            if ($job == null) {
                return failedResponse('Not Found job', Response::HTTP_NOT_FOUND);
            }
            $jobId = $job->toArray()['id'];
        }

        $data = $this->repository->createJob($client, $jobRequest->validated(), $jobId);
//        return $data;
        return successResponse(true,Job::find($data)->load('skill') , 'add successfully');
    }

    public function update(UpdateJobRequest $jobRequest,$id)
    {
        $jobs=Job::find($id);
        if (!$jobs)
            return $this->onFaile('Job not found');

        if (array_key_exists('job', $jobRequest->validated())) {
            $jobTitleRepository = new JobTitleRepository();
            $job = $jobTitleRepository->findJob($jobRequest->validated()['job']);
//            return $job;
            if ($job == null) {
                return failedResponse('industry Not Found', Response::HTTP_OK);

            }
            $jobId = $job->toArray()['id'];

//            return $jobId;


        }
        $JobTitle=   SkillJob::where('job_id',$id)->get();

        Skill::whereIn('id',$JobTitle->pluck('skill_id'))->delete() ;
        SkillJob::where('job_id',$id)->delete();
//        return $jobs;
        ;
//        if ()
        if (array_key_exists('skill', $jobRequest->validated())) {

            foreach ($jobRequest['skill'] as $item){
            $datas[]= Skill::query()->create([
                'name'=>$item
            ])->id;
        }
            $jobs->skill()->sync($datas);

        }
        $data = $jobs->update(['status'=>$jobRequest->status,'job_title_id'=>$jobId,'experience'=>$jobRequest->experience,'description'=>$jobRequest->description,'phone_number'=>$jobRequest->phone_number]);
//            return $data;
//               return Job::find($id);

        return ($data)?$this->successResponseWithoutMessage(true,   Job::with(['job_title'=>function($q){
            $q->select('title','id');

        },'skill'=>function($q){
            $q->select('name');
        }])->first()):$this->failedResponse('job not find');
    }

    public function destroy($id)
    {
        $repo = $this->repository->deleteJob($id);
        return ($repo)?$this->successResponse(true, null, 'delete job successfully')
            :$this->failedResponse('not find job delete ');
    }

    public function index()
    {
        $repo = $this->pipeline(Job::query()->with('skill','job_title'),[JobTitle::class]);
        return $this->successResponse(true, $repo, 'send FAQ successfully');
    }

    public function indexForuser()
    {
        $repo = $this->repository->showFAQ();
        return $this->successResponse(true, $repo, 'send FAQ successfully');
    }

}
