<?php

namespace App\Http\Controllers;

use App\Helper\ApiResponserHelper;
use App\Http\Requests\EmailRequest;
use App\Mail\ConfirmMail;
use App\Mail\EstimationMail;
use App\Mail\ForgotPasswordMail;
use App\Mail\MessageMail;
use App\Models\Company;
use App\Models\User;
use App\Repositories\Role\SearchRepository;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;


class EmailController extends Controller
{
    public $request;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    public function sendEmailConfirmation(EmailRequest $emailRequest, SearchRepository $searchRepository)
    {
        //check email exists
        if ($searchRepository->searchEmail(User::class, $this->request['email'])
            || $searchRepository->searchEmail(Company::class, $this->request['email']))
        {
            return successResponse(false, null, 'your email is used before');
        }
        Mail::to($this->request['email'])->send(new ConfirmMail($this->request['email']));
        return successResponse(true, null, "Sent code to your email successfully");
    }

    public function sendEmailResetPassword(SearchRepository $searchRepository)
    {
        $email = $this->request['email'];
//        return $email;
        if (!!$searchRepository->searchEmail(User::class, $email)
            || !!$searchRepository->searchEmail(Company::class, $email))
        {
            //make confirm code
            $confirmationCode = rand(10000, 99999);
            $time = Carbon::now()->format('d F Y');
            $value = Cache::put($email, $confirmationCode, now()->addMinute(3));
//return $time;
            //send email

            Mail::to($email)->send(new ForgotPasswordMail($confirmationCode,$time));
        }
        return successResponse(true, null, "If an account exists for $email , you will get an email for resetting your password.");
    }

    public function sendEmailMessage($email, $emailSender, $message)
    {
        $time = Carbon::now()->format('d F Y');
        Mail::to($email)->send(new MessageMail($emailSender, $message, $time));
        return ApiResponserHelper::successResponse(true, null, 'send email successfully');
    }

    public function sendEstimateEmail($data, $pdf)
    {
        $data['email'] = $data['customer_email'];
        $data['title'] = $data['subject'];
//       $rand= rand(9999999999,99999999);
        Mail::send('mail.Estimation', $data, function ($message) use ($data, $pdf) {
            $message->from(env('FROM_EMAIL'), $data['title'])
                ->to($data["email"], $data["email"])
                ->subject($data["title"])
                ->attachData($pdf['pdf']->output(), Storage::disk('public')->url($pdf['path']));
        });
    }
    public function reSendEstimateEmail($data, $pdf)
    {
//        return $data;
        $arrData = [
            'email'=> $data['customer_email'],
            'title'=>$data['subject'],
            'body'=>'estimte',
            'pdf_address'=>$data['pdf_address']
        ];

//       $rand= rand(9999999999,99999999);
        Mail::send('mail.Estimation', $arrData, function ($message) use ($arrData, $pdf) {
            $message->from(env('FROM_EMAIL'), $arrData['title'])
                ->to($arrData["email"], $arrData["email"])
                ->subject($arrData["title"]);
//                ->attachData(Storage::disk('public')->get($arrData['pdf_address']), Storage::disk('public')->url($pdf['path']));
        });
    }
    public function sendInvoiceEmail($data, $pdf)
    {
        $data['email'] = $data['customer_email'];
        $data['title'] = $data['subject'];
//        $rand= rand(9999999999,99999999);
        Mail::send('mail.Estimation', $data, function ($message) use ($data, $pdf) {
            $message->from(env('FROM_EMAIL'), $data['title'])
                ->to($data["email"], $data["email"])
                ->subject($data["title"])
                ->attachData($pdf['pdf']->output(), Storage::disk('public')->url($pdf['path']));
        });
    }

    public function sendExternalEstimate($email, $data)
    {
//        return $email;
        foreach ($email as $item) {
            Mail::to($item)->send(new EstimationMail($email, $data));
        }
    }
    public function sendExternalEstimate2($email, $data,$pdf)
    {
//        return $pdf;
        Mail::send('mail.ExternalEstimation', $data, function ($message) use ($data, $pdf,$email) {
            $message->from(env('FROM_EMAIL'), $data['subject'])
                ->to($email, $email)
                ->subject($data["subject"])
                ->attachData($pdf->output(),rand(9999999999999,999999999999));
        });
    }
}
