<?php

namespace App\Http\Controllers;

use App\Filter\FilterClass\Map\CompanyProject;
use App\Filter\FilterClass\Map\Priority;
use App\Filter\FilterClass\project\Address;
use App\Filter\FilterClass\project\Condition;
use App\Filter\FilterClass\project\Industry;
use App\Filter\FilterClass\project\participants\Name;
use App\Filter\FilterClass\project\ZipCode;
use App\Http\Resources\MapHomeResource;
use App\Http\Resources\PaginationResource;
use App\Models\Admin\Structure\SiteInformation;
use App\Models\Admin\Structure\Team;
use App\Models\Company;
use App\Models\HomeText;
use App\Models\Project;
use App\Models\User;
use App\Repositories\Admin\Structure\FAQRepository;
use App\Repositories\Admin\Structure\SiteInformationRepository;
use App\Repositories\Admin\Structure\TeamRepository;
use App\Repositories\JobTitleRepository;
use App\Repositories\ProjectRepository;
use App\Repositories\Role\SearchRepository;
use App\Repositories\Role\SettingInformationRepository;
use App\Services\MapService;
use App\Traits\ApiResponser;
use App\Traits\pipeline;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;


class   HomeController extends Controller
{
    use ApiResponser,pipeline;

    public function count(ProjectRepository $projectRepository, SettingInformationRepository $settingInformationRepository)
    {
        $project = $projectRepository->countAll();
        $user = $settingInformationRepository->countUser();
        $company = $settingInformationRepository->countCompany();
        $users = $user + $company;
        return $this->successResponse(true, ['project' => $project, 'users' => $users], "send successfully");
    }

    public function getSpecialProject(ProjectRepository $projectRepository)
    {
        $project = $projectRepository->findSpecial();
        return $this->successResponse(true, $project, 'send successfully');
    }

    public function getSiteInformation(SiteInformationRepository $siteInformationRepository, Request $request)
    {
        $information = $siteInformationRepository->findByType($request['type']);
        if ($information->toArray() == null) {
            return $this->successResponse(false, null, 'not found key');
        }
        return $this->successResponse(true, $information, 'send successfully');
    }

    public function getTeam(TeamRepository $teamRepository)
    {
        $team = $teamRepository->showAll();

        if ($team->toArray() == null) {
            return $this->successResponse(false, null, 'not found any data');
        }
        return $this->successResponse(true, $team, 'send successfully');
    }


    public function getProjectOnMAp(Request $request)
    {
        $mapService = new MapService();

        $projects = Project::with('address')->get();
        foreach ($projects as $item) {
            if ($item->address['latitude'] != null && $lon2 = $item->address['longitude'] != null) {
                $lat2 = $item->address['latitude'];
                $lon2 = $item->address['longitude'];
                $des = $mapService->distance($request->latitude, $request->longitude, $lat2, $lon2);
                if ($request['destination'] >= $des){
                    $projectList[] = $item;
                }
            }
        }
        return successResponse(true, $projectList, 'send successfully');
    }


    public function getClientByJobTitle($jobTitle, JobTitleRepository $jobTitleRepository, SearchRepository
$searchRepository)
    {
        $job = $jobTitleRepository->findJob($jobTitle);
        if ($job == null){
            return failedResponse('Not Found job', Response::HTTP_NOT_FOUND);
        }
        $roles = [User::class, Company::class];
         foreach ($roles as $role){
             $list[]  = $searchRepository->searchJobTitle($role, $job->id);
         }

         return $this->successResponse(true,$list, ' ok');
    }
    public function indexForUser()
    { $repositoryFaq=new FAQRepository;
        $repo = $repositoryFaq->showFAQ();
        return ($repo)?$this->successResponse(true, $repo, 'send FAQ successfully'):
            $this->failedResponse('failed delete FAQ ');
    }
    public function footer(){
        $home['setting']=['link'=>[
            'instagram'=>SiteInformation::where('key',SiteInformation::INSTAGRAM)->first()->value,
            'twitter'=>SiteInformation::where('key',SiteInformation::TWITTER)->first()->value,
            'linkedin'=>SiteInformation::where('key',SiteInformation::TWITTER)->first()->value,
            'facebook'=>SiteInformation::where('key',SiteInformation::FACEBOOK)->first()->value,
        ],
            'description'=>SiteInformation::where('key',SiteInformation::SITE_DESCRIPTION)->first()->value
            ,'address'=>SiteInformation::where('key',SiteInformation::SITE_ADDRESS)->first()->value,
            'one_number'=>SiteInformation::where('key',SiteInformation::MAINE_PHOTO_NUMBER)->first()->value,
            'two_number'=>SiteInformation::where('key',SiteInformation::SECOND_PHOTO_NUMBER)->first()->value,
            'Three_number'=>SiteInformation::where('key',SiteInformation::Third_PHOTO_NUMBER)->first()->value,
            'users_count'=>User::count()+Company::count()
        ];
        return $this->successResponseWithoutMessage(true,$home['setting']);
    }
    public function home(){
        $home['count']=[
            'user_count'=>User::count()+Company::count()  ,
            'group_count'=>DB::table('chatable_groups')->count(),
            'project_count'=>Project::count(),
        ] ;
//        $map=Project::with(['address' => fn($query)=>$query->select('id')->select('id','name')->get()]);
        $map=Project::with(['address'])->select('id','name')->get();
        $home['map']=MapHomeResource::collection($map);
        $home['text']=HomeText::select('content')->get();
        $home['link']=['google_play'=>SiteInformation::where('key',SiteInformation::LINK_APP_GOOGLE_PLAY)->first()->key??null,'app_store'=>SiteInformation::where('key',SiteInformation::LINK_APP_STORE)->first()->key??null];
        return ($home)? $this->onSuccess($home):$this->onFaile(' not find');
    }
    public function searchHome(){
//        return 5;
//        return (Project::query()->with('address','participants.participantsProject')->get());

    $project=$this->pipeline(Project::query()->with('address'),[Name::class,ZipCode::class,Priority::class,CompanyProject::class,Address::class,Industry::class ]);
      return ($project)?$this->onSuccess(MapHomeResource::collection($project)):$this->onFaile('project no found ');
    }
    public function searchHomeForApp(){
//        return 2;
        $project=$this->pipeline(Project::query()->with('address'),[Name::class,ZipCode::class,Priority::class,CompanyProject::class]);
//        return $project;
//        dd($project);
        return ($project)?$this->onSuccess(new PaginationResource($project,MapHomeResource::collection($project))):$this->onFaile('project no found ');
    }
    public function aboutUs(){
    $about['about']=
        ['top_title'=>SiteInformation::ABOUT_US_MAIN_PARAGRAPH_TITLE,SiteInformation::where('key',SiteInformation::ABOUT_US_MAIN_PARAGRAPH_TITLE)->first()->value ??null,
         'top_text'=>SiteInformation::where('key',SiteInformation::ABOUT_US_MAIN_PARAGRAPH)->first()->value??null,
          'down_title'=>SiteInformation::where('key',SiteInformation::ABOUT_US_SECONDARY_TEXT_TITLE)->first()->value??null,
          'down_text'=>SiteInformation::where('key',SiteInformation::ABOUT_US_SECONDARY_TEXT)->first()->value??null,
        ];
        $about['teme']=Team::select('first_name','last_name','job','image','about_us')->get();
    return isset($about['teme'])? $this->onSuccess($about) :$this->onFaile(' not find');
    }
    public function support(){
        $about['support']=
            [
                'address'=>SiteInformation::where('key',SiteInformation::ABOUT_US_MAIN_PARAGRAPH_TITLE)->first()->value??null,
                'phone_one'=>SiteInformation::where('key',SiteInformation::MAINE_PHOTO_NUMBER)->first()->value??null,
                'phone_two'=>SiteInformation::where('key',SiteInformation::SECOND_PHOTO_NUMBER)->first()->value ??null,
                'phone_three'=> SiteInformation::where('key',SiteInformation::Third_PHOTO_NUMBER)->first()->value ??null,
                'email'=>SiteInformation::where('key',SiteInformation::EMAIL)->first()->value ??null,
                'twitter'=>SiteInformation::where('key',SiteInformation::TWITTER)->first()->value??null,
                'instagram'=>SiteInformation::where('key',SiteInformation::INSTAGRAM)->first()->value??null,
                'linkedin'=>SiteInformation::where('key',SiteInformation::LINKEDIN)->first()->value ??null,
                'facebook'=>SiteInformation::where('key',SiteInformation::FACEBOOK)->first()->value??null ,
            ];
        return isset(  $about['support'])?$this->onSuccess($about['support'])  :$this->onFaile(' not find');
    }
    public function Terms(){
        $about['terms']= SiteInformation::where('key',SiteInformation::TERMS)->first()->value??null;
        return (isset( $about['terms'])) ?$this->onSuccess($about['terms']) :$this->onFaile('Terms not find');
    }

}
