<?php

namespace App\Models;

use App\Events\ChatEvent;
use App\Models\Admin\Structure\BillingPlan;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Storage;
use Laravel\Sanctum\HasApiTokens;
use Mojtaba\Chatable\Models\Message as Messages;
use Mojtaba\Chatable\Traits\Chatable;

class Company extends Authenticatable
{
    use HasFactory, HasApiTokens, Chatable, Notifiable,SoftDeletes;

    protected $table = 'companies';
    protected $guarded = [
        'id'
    ];
protected $appends=['type','role_client'];

    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */

    protected $casts = [
        'email_verified_at' => 'datetime',
        'start_date' => 'datetime',

    ];

    public function jobTitle()
    {
        return $this->belongsTo(JobTitle::class);
    }

    public function owner()
    {
        return $this->morphOne(Owner::class, 'ownerable');
    }

    public function address()
    {
        return $this->morphOne(Address::class, 'addressable', "addressable_type", "addressable_id");
    }

    public function socialMedias()
    {
        return $this->morphToMany(SocialMedia::class, 'social_mediable');
    }

    public function reports()
    {
        return $this->morphMany(Report::class, 'reportable');
    }

    public function getImageAttribute($value)
    {
        return $value != null ? Storage::disk('public')->url('images/' . $value) : $value;
    }

    public function projects()
    {
        return $this->morphMany(Project::class, 'projectable');
    }

    public function participantProjects()
    {
        return $this->morphToMany(Project::class, 'participant_project');
    }

    public function messages()
    {
        return $this->morphOne(Message::class, "messageable");
    }

    public function subscribers()
    {
        return $this->morphToMany(BillingPlan::class, 'client');
    }

    public function transactions()
    {
        return $this->morphMany(Transaction::class, 'client');
    }

    public function invitations()
    {
        return $this->morphMany(Invite::class, 'client');
    }

    public function inviter()
    {
        return $this->morphMany(Invite::class, 'inviter');
    }

    public function countProject()
    {
        return $this->morphOne(CountProject::class, 'client');
    }

    public function members()
    {
        return $this->morphMany(Member::class, 'member');
    }

    /*public function sendChatMessageEvent(Messages $message)
    {
        event(new ChatEvent($message));
    }*/

    public function images()
    {
        return $this->morphMany(Image::class, 'client');
    }

    public function sendChatMessageEvent(Messages $message)
    {
        event(new ChatEvent($message));
    }

    public function estimates()
    {
        return $this->morphMany(Estimate::class, 'estimateable');
    }

    public function invoices()
    {
        return $this->morphMany(Invoice::class, 'invoiceable');
    }

    public function customers()
    {
        return $this->morphMany(Customer::class, 'customerable');
    }

    public function clientList()
    {
        return $this->morphMany(ServiceList::class, 'clientList');
    }

    public function clientComments()
    {
        return $this->morphMany(Comment::class, 'client');
    }

    public function profileComments()
    {
        return $this->morphMany(Comment::class, 'profile');
    }

    public function clientJobs()
    {
        return $this->morphMany(Job::class, 'client');
    }
    public function rates()
    {
        return $this->morphOne(Rate::class, 'rateable','rateable_type','rateable_id');
    }
    public function industry(){
        return $this->hasOne(JobTitle::class,'job_title_id','Id');
    }
    public function getTypeAttribute(){
        return 'company';
    }
   public function getRoleClientAttribute(){
        return 'company';
    }
    public function participantsProject()
    {
        return $this->morphToMany(Project::class, 'participants_project','participants_project');
    }
    public function specialCommentsClient()
    {
        return $this->morphOne(SpecialComment::class, 'client');
    }
    public function specialCommentsProfile()
    {
        return $this->morphOne(SpecialComment::class, 'profile');
    }

}
