<?php

namespace App\Services;

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);


use App\Http\Controllers\KeywordController;
use App\Models\Image;
use App\Models\User;
use App\Traits\ApiResponser;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Intervention\Image\ImageManager;

class ImageService
{
    use ApiResponser;

    private $imageManager;

//    public function __construct(
//        ImageManager $imageManager
//    ){
//        $this->imageManager = $imageManager;
//    }

    public function combineImages($firstImage, $secondImage)
    {
        $firstImageDetail = getimagesize($firstImage);
        $firstImageWidth = $firstImageDetail[0];
        $firstImageHeight = $firstImageDetail[1];

        $secondImageDetail = getimagesize($secondImage);
        $secondImageWidth = $secondImageDetail[0];
        $secondImageHeight = $secondImageDetail[1];

        $width = $firstImageWidth + $secondImageWidth;

        if ($firstImageHeight != $secondImageHeight){
            return 0;
        }
        //$height = $firstImageHeight >= $secondImageHeight ? $firstImageHeight : $secondImageHeight;
        $height = $firstImageHeight;

        $image = imagecreatetruecolor($width, $height);

        $firstImageName = time() . Str::random(20) . '.' . $firstImage->extension();
        Storage::drive('public')->putFileAs("temp/images",$firstImage ,$firstImageName);

        $secondImageName = time() . Str::random(20) . '.' . $secondImage->extension();
        Storage::drive('public')->putFileAs("temp/images",$secondImageName ,$firstImageName);


        $imageA = imagecreatefromjpeg(public_path("/storage/temp/images/$firstImageName"));

        imagecopyresampled($image, $imageA, 0, 0, 0, 0, $width, $width, $width, $width);

        ob_clean();
        header("Content-Type: image/jpg");

        $imageB = imagecreatefromjpeg(public_path("/storage/temp/images/$secondImageName"));


        imagecopyresampled($image, $imageB, $firstImageWidth, 0, 0, 0, $width, $width, $width, $width);
        $imageName = time() . Str::random(20);

        imagejpeg($image, public_path('/storage/temp/images/' . "$imageName" . '.jpg'));

        //delete image that dont use
        File::delete(public_path("/storage/temp/images/$secondImageName"));
        File::delete(public_path("/storage/temp/images/$firstImageName"));

        return $imageName .'.jpeg'; //return image path
    }

    public function combineImagesWithPackage($firstImage, $secondImage){
        $imageManager=new ImageManager ;
        $storage= Storage::disk('public')->putFile('temp/images',$firstImage);
        $storageSecondImage= Storage::disk('public')->putFile('temp/images',$secondImage);
        $img_left_local =storage_path('app/public/'.$storage);
        $img_right_local =storage_path('app/public/'.$storageSecondImage);
        $imageMakerOne = \Intervention\Image\Facades\Image::make($img_left_local);
        $imageMakerTwo = \Intervention\Image\Facades\Image::make($img_right_local);
        $img_canvas = $imageManager->canvas($imageMakerOne->getWidth() + $imageMakerTwo->getWidth() + 5,($imageMakerOne->getHeight() > $imageMakerTwo->getHeight()) ? $imageMakerOne->getHeight() : $imageMakerTwo->getHeight());
        $img_canvas->insert($imageMakerOne, 'top-left');
        $img_canvas->insert($imageMakerTwo, 'top-right');
        $pathImage="/temp/images/".rand(9999,22)."_vs_".rand(999,22).".jpg";
       $path= storage_path("app/public".$pathImage);
         $img_canvas->save($path, 100);
         return Storage::disk('public')->url($pathImage);
    }
    public function storePhoto($imageData){
        if ($imageData)
            return Storage::disk('public')->putFile('temp/images',$imageData);
    }

    public function combineImagesssss($firstImage, $secondImage)
    {
        $width = 1364;
        $height = 681;

        $image = imagecreate($width, $height) or die('cannot do any things');
        //$canvas = imagecolorallocate($image, 200, 6, 245);

        $imageA = imagecreatefromjpeg(public_path('/storage/temp/images/1.jpeg'));

        imagecopyresampled($image, $imageA, 0, 0, 0, 0, 682, 682, 682, 682);
        ob_clean();
        header("Content-Type: image/jpg");

        $imageB = imagecreatefromjpeg(public_path('/storage/temp/images/2.jpeg'));
        imagecopyresampled($image, $imageB, 682, 0, 0, 0, 682, 682, 682, 682);
        $imageName = time() . Str::random(20);

        imagejpeg($image, public_path('/storage/temp/images/' . "$imageName" . '.jpeg'));

        return 1; //return image path
    }

    public function uploadToPublic($image)
    {
        $imageName = time() . Str::random(20) . '.' . $image->extension();
        Storage::putFileAs("public/images", $image, $imageName);
        return $imageName;
    }

    public function moveTempToPublic($imageName)
    {

        Storage::move("temp/images/" . $imageName, "public/images/" . $imageName);
        return 1;
    }

    public function checkExist($imageName)
    {
        return Storage::exists("temp/images/$imageName");
    }


    public function showUserImage($id)
    {
        $imageName = User::select('image')->where('id', $id)->get();
        $imagePath = '/public/images' . $imageName;
        $url = Storage::url($imageName);
        dd($url);
    }

    public function insertKeywordImage($images, $keyword)
    {
        $keywordController = new KeywordController();
        $keywords = $keywordController->findOrCreate($keyword);
        $keywordId = $keywords->toArray()['id'];
        return $keywordId;
    }

    public function insertImage($imageName, $type, $id, $clientType, $clientId, $link = null)
    {
        return Image::create([
            'name' => $imageName,
            'website_link' => $link,
            'imageable_id' => $id,
            'imageable_type' => $type,
            'client_type' => $clientType,
            'client_id' => $clientId
        ]);
    }

    public function insertProjectImage($image, $project)
    {
        return DB::table('image_project')->insert([
            'image_id' => $image,
            'project_id' => $project
        ]);
    }

    public function showProjectImage($id)
    {
        DB::table('image_project')->where('project_id', $id)->get();
    }


    public function showReportImage($id)
    {
        DB::table('image_report')->where('report_id', $id)->get();
    }

    public function addTagToImage($imageId, $tagId)
    {

    }
}
